
 /* for the numerical integration
   method presented by professor Everhart E. celestial mech., 1974, v.10 */

/* unit uneverim   global variables */

#include <math.h>

#include "uneverim.h" 
#include "unequras.h" /* for the void clcrighthandcoor */

   int    gratofla;        /* for the first call 09.11.2008 */
   double steprada[9];
   double dimcoefr[9];
   double dimcoefv[9];
   double coefever[9][4];
   double everalfa[9][4];
   double alphamom[3][5];
   int    memalpha;

   double dift[9];
   double difd[9];
   double coef[9][9];

 void everinit ( ) /* initial value to some variables */
 {
   steprada[1] = 0.000000000000000000;
   steprada[2] = 0.056262560526922147;
   steprada[3] = 0.180240691736892365;
   steprada[4] = 0.352624717113169637;
   steprada[5] = 0.547153626330555383;
   steprada[6] = 0.734210177215410532;
   steprada[7] = 0.885320946839095768;
   steprada[8] = 0.977520613561287501;
   dimcoefr[1] = 0.500000000000000000;
   dimcoefr[2] = 0.166666666666666667;
   dimcoefr[3] = 0.083333333333333333;
   dimcoefr[4] = 0.050000000000000000;
   dimcoefr[5] = 0.033333333333333333;
   dimcoefr[6] = 0.023809523809523810;
   dimcoefr[7] = 0.017857142857142857;
   dimcoefr[8] = 0.013888888888888889;
   dimcoefv[1] = 1.000000000000000000;
   dimcoefv[2] = 0.500000000000000000;
   dimcoefv[3] = 0.333333333333333333;
   dimcoefv[4] = 0.250000000000000000;
   dimcoefv[5] = 0.200000000000000000;
   dimcoefv[6] = 0.166666666666666667;
   dimcoefv[7] = 0.142857142857142857;
   dimcoefv[8] = 0.125000000000000000;
 }

 void evercoefnullo ( )  /* initial moment nullo to start */
 {
   int i,j ;

   gratofla = 0;  /* to start integration */

   for (  i = 1 ;  i <= 8 ; i++ )
     for (  j = 1 ;  j <= 3 ; j++ )
       {
         coefever[i][j] = 0.0;
         everalfa[i][j] = 0.0;
       }
   memalpha = 0;
   for (  i = 1 ;  i <= 2 ; i++ )
     for (  j = 1 ;  j <= 4 ; j++ )
       alphamom[i][j] = 0.0;
 }

 /* to obtain vector
      on the moment  t  from the coefficients of the polinoms
      constructed in integrator process presented by professor everhart */


 void clcvectfrompolinom ( double dstep ,
                           double zeror[4] , double zerov[4] ,
                           double vectr[4] , double vectv[4] )
 { /* dstep in day of time from nullo point */
   int    i,j ;
   double curr[4] ;
   double curv[4] ;
 
   for (  j = 1 ;  j <= 3 ; j++ )
     {
       curv[j] = dimcoefv[8]*coefever[8][j];
       curr[j] = dimcoefr[8]*coefever[8][j];
     }
   for (  i = 7;  i >= 1; i-- )
     for (  j = 1 ;  j <= 3 ; j++ )
       {
         curv[j] = dimcoefv[i]*coefever[i][j]+curv[j]*dstep;
         curr[j] = dimcoefr[i]*coefever[i][j]+curr[j]*dstep;
       }
   for (  j = 1 ;  j <= 3 ; j++ )
     {
       vectv[j] = zerov[j]+curv[j]*dstep;
       vectr[j] = zeror[j]+(zerov[j]+curr[j]*dstep)*dstep;
     }
 }

 void fromprevious ( double *tbeg , double zeror[4] , double zerov[4] ,
                     double *tend , double vectr[4] , double vectv[4] )
 {
   int i ;
 
   *tbeg = *tend; /* to change condition from previous step */
   for (  i = 1 ;  i <= 3 ; i++ )
     {
       zeror[i] = vectr[i];
       zerov[i] = vectv[i];
     }
 }

 void coefromrecur ( double stepfull )
 {
   int i,j ;

   dift[1] = 0; /* stepfull in day */
   for (  i = 2 ;  i <= 8 ; i++ )
     {
       dift[i] = steprada[i]*stepfull;
       difd[i] = dift[i];
     }
   for (  i = 1 ;  i <= 8 ; i++ )
     for (  j = 1 ;  j <= 8 ; j++ )
       coef[i][j] = 0.0;
   /* the simple coefficients from recurrence formula */
   for (  i = 1 ;  i <= 8 ; i++ )
     coef[i][i] = 1.0;
   for (  i = 2 ;  i <= 8 ; i++ )
     coef[i][1] = -dift[i]*coef[i-1][1];
   for (  i = 3 ;  i <= 8 ; i++ )
     for (  j = 2 ;  j <= i-1 ; j++ )
       coef[i][j] = coef[i-1][j-1]-dift[i]*coef[i-1][j];
 }

 void coefromalpha ( int icur )
 {
   int ia,j,k ;

   for (  j = 1 ;  j <= 3 ; j++ )
     for (  ia = 2 ;  ia <= icur ; ia++ )
       {
         coefever[ia][j] = everalfa[ia][j];
         for (  k = ia+1 ;  k <= 8 ; k++ )
           coefever[ia][j] = coefever[ia][j]
                           +coef[k-1][ia-1]*everalfa[k][j];
       }
 }

 void tochangealpha ( double tbeg )
 {
   int    j ;
   double d ;
 
   if  ( memalpha < 2 ) return ; /* no action no alpha values in memory */
   d = (tbeg-alphamom[2][4])/(alphamom[2][4]-alphamom[1][4]);
   for (  j = 1 ;  j <= 3 ; j++ )
     everalfa[2][j] = alphamom[2][j]+(alphamom[2][j]-alphamom[1][j])*d;
   coefromalpha(2);  /* to improve the value of the second coefficients */
 }

 void alphavaluetomemory ( double tbeg )
 {
   int j ;

   memalpha = memalpha+1;
   if  ( memalpha == 3 )
       {
         memalpha = 2;
         for (  j = 1 ;  j <= 3 ; j++ )
           alphamom[1][j] = alphamom[2][j];
         alphamom[1][4] = alphamom[2][4];
       }
   for (  j = 1 ;  j <= 3 ; j++ )
     alphamom[memalpha][j] = everalfa[2][j];
   alphamom[memalpha][4] = tbeg;
 }

 /* to integrate the full function
      by the use of the method presented by professor everhart */

 void posvelevergrator ( double stepfull ,
                         double *tbeg , double zeror[4] , double zerov[4] ,
                         double *tend , double vectr[4] , double vectv[4] )
 {
   int    i,j,ia,it ;
   double tc,dst,dt ;
   double zbeg,zend ;
   double funb[4],func[4] ;
   double rcur[4],vcur[4] ;

   zbeg = *tbeg;
   zend = *tend;
   fromprevious(&zbeg,zeror,zerov,&zend,vectr,vectv);/* to new condition */
   *tbeg = zbeg;
   *tend = zend;

   if  ( fabs(stepfull) < 1.0e-12 ) return ; /* no step needs */

   coefromrecur(stepfull); /* stepfull in day */

   /* value of the forces at the first point  t0 */
   clcrighthandcoor(zbeg,zeror,funb); /* tbeg in mod. julian day */

   /* initial values of the coefficients of the polinoms for forces */
   for (  j = 1 ;  j <= 3 ; j++ )
     coefever[1][j] = funb[j]; /* the first coefficients */

   tochangealpha(zbeg); /* if there are two previous alpha values */


 for ( it = 1 ; it <= 4 ; it++ )
  {
   for (  i = 2 ;  i <= 8 ; i++ )
     {
       dst = dift[i];
       clcvectfrompolinom(dst,zeror,zerov,rcur,vcur);

       tc = zbeg+difd[i]; /* step in da tcur in mod. julian day */

       /* value of the forces at the next point i */
       clcrighthandcoor(tc,rcur,func); /* from unforces */

       dt = 1.0/dift[i];
       for (  j = 1 ;  j <= 3 ; j++ )
         everalfa[i][j] = (func[j]-funb[j])*dt;

       /* to get the next alpha value */
       for (  ia = 3 ;  ia <= i ; ia++ )
         {
           dt = 1.0/(dift[i]-dift[ia-1]);
           for (  j = 1 ;  j <= 3 ; j++ )
             everalfa[i][j] = (everalfa[i][j]-everalfa[ia-1][j])*dt;
         }

       coefromalpha(i);  /* to improve the value of coefficients */

     }
  }

   /* coefficients of the polinoms have been calculated
     it is possible to obtain the vectors at the last point */
   *tend = *tbeg+stepfull; /* stepfull in day */
   clcvectfrompolinom(stepfull,zeror,zerov,vectr,vectv);

   alphavaluetomemory(zbeg) ;

 }


 /* it is possible to change step by using coefever[8][1..3] */

 void fornextstep ( double  zo ,  /* step before */
                    double *zt )  /* step  after */
 {
   int     i ; /* count for coordinates */
   double  o ; /* input step */
   double  z ; /* temporary value */
   double  q ; /* temporary value */
   double  d ; /* output step */

   o = zo ;  /* input step in day */
   /* to estimate suitable step for the next */
   z = 0.0;
   for (  i = 1 ;  i <= 3 ; i++ )
     {
       q = fabs(coefever[4][i]); /* temporary value |a_4| */
       if  ( z < q )  z = q;     /* to find maximum value */
     }
   q = fabs(o);       /* temporary value step may be negative */
   q = pow(q,4.0);    /* temporary step^4 in power 4 */
   z = z/(30.0*q);    /* temporary value to estimate */
   if  ( z == 0.0 )
         d = 1.4*o; /* step after is larger */
   else
       {
         q = 1.0e-10/z;           /* with accuracy parameter L=10 */
         d = pow(q,1.0/9.0);      /* simple estimate */
         if  ( o < 0.0 )          /* step may be negative */
             d = -d;              /* new step is negative */
         if  ( fabs(d/o) > 1.4 )  /* new step may be too large */
             d = 1.4*o;           /* to reduce with coefficient */
       }

    *zt = d; /* output value for new step */
 }

 /* to integrate the full function with variable step
      by the use of the method presented by professor everhart */

 void posvelevergratos ( double *stepfull ,
                         double *tbeg , double zeror[4] , double zerov[4] ,
                         double *tend , double vectr[4] , double vectv[4] )
 {
   int    i,j,ia,it ;
   int    maxiter ;
   double tc,dst,dt ;
   double zbeg,zend ;
   double funb[4],func[4] ;
   double rcur[4],vcur[4] ;
   double stepbefor ;
   double stepafter ;

   zbeg = *tbeg;
   zend = *tend;
   fromprevious(&zbeg,zeror,zerov,&zend,vectr,vectv);/* to new condition */
   *tbeg = zbeg;
   *tend = zend;

   if  ( fabs(*stepfull) < 1.0e-12 ) return ; /* no step needs */

   if  ( gratofla == 0 )
       {
         maxiter = 6 ;  /* initial number of iterations */
         gratofla = 1 ; /* to change for the next step */
       }
   else  maxiter = 2;   /* for the next step */

   stepbefor = *stepfull; /* input step to integrate */
      

   coefromrecur(stepbefor); /* stepbefor in day */

   /* value of the forces at the first point  t0 */
   clcrighthandcoor(zbeg,zeror,funb); /* tbeg in mod. julian day */

   /* initial values of the coefficients of the polinoms for forces */
   for (  j = 1 ;  j <= 3 ; j++ )
     coefever[1][j] = funb[j]; /* the first coefficients */

   tochangealpha(zbeg); /* if there are two previous alpha values */


 for ( it = 1 ; it <= maxiter ; it++ )
  {
   for (  i = 2 ;  i <= 8 ; i++ )
     {
       dst = dift[i];
       clcvectfrompolinom(dst,zeror,zerov,rcur,vcur);

       tc = zbeg+difd[i]; /* step in da tcur in mod. julian day */

       /* value of the forces at the next point i */
       clcrighthandcoor(tc,rcur,func); /* from unforces */

       dt = 1.0/dift[i];
       for (  j = 1 ;  j <= 3 ; j++ )
         everalfa[i][j] = (func[j]-funb[j])*dt;

       /* to get the next alpha value */
       for (  ia = 3 ;  ia <= i ; ia++ )
         {
           dt = 1.0/(dift[i]-dift[ia-1]);
           for (  j = 1 ;  j <= 3 ; j++ )
             everalfa[i][j] = (everalfa[i][j]-everalfa[ia-1][j])*dt;
         }

       coefromalpha(i);  /* to improve the value of coefficients */

     }
  }

   /* coefficients of the polinoms have been calculated
     it is possible to obtain the vectors at the last point */
   *tend = *tbeg+stepbefor; /* stepbefor in day */
   clcvectfrompolinom(stepbefor,zeror,zerov,vectr,vectv);

   alphavaluetomemory(zbeg) ;

   fornextstep(stepbefor,&stepafter); /* it is possible to change step */

   *stepfull = stepafter;

 }
