
/* unit unfornut  constant */

#include <math.h>

#include "unfornut.h"

 double togetepsmean ( double tmjd )
 {
   double dt,dt2,dt3 ;
   double secrad ;

   secrad = M_PI/(180.0*3600.0) ;
   dt = (tmjd-51544.5e0)/36525.0e0;
   dt2 = dt*dt;
   dt3 = dt*dt2;
   return secrad*(84381.448-46.8150*dt-0.00059*dt2+0.001813*dt3) ;
 }

 /* clcfundarg is a void
   to calculate fundamental arguments in radians
      on moment tmjd in modified julian date.
      fundarg[1..5] is massive of the values of arguments.
      global const - grarad - for change from degrees to radians. */

 void clcfundarg ( double tmjd , double fundarg[6] )
 {
   double dt,dt2,dt3 ;
   double grarad ;
   int    i ;

   grarad = M_PI/180.0 ;
   dt = (tmjd-51544.5e0)/36525.0e0;
   dt2 = dt*dt;
   dt3 = dt*dt2;
   fundarg[1] = 218.31643250e0+481267.8812772222e0*dt
                             -0.00161167e0*dt2+0.00000528e0*dt3;
   fundarg[2] = 134.96298139e0+477198.8673980556e0*dt
                             +0.00869722e0*dt2+0.00001778e0*dt3;
   fundarg[3] = 357.52772333e0+35999.05034e0*dt
                             -0.00016028e0*dt2-0.00000333e0*dt3;
   fundarg[4] =  93.27191028e0+483202.0175380555e0*dt
                             -0.00368250e0*dt2+0.00000306e0*dt3;
   fundarg[5] = 297.85036306e0+445267.11148e0*dt
                             -0.00191417e0*dt2+0.00000528e0*dt3;
   for (  i = 1 ;  i <= 5 ; i++ )
     fundarg[i] = grarad*fundarg[i];
 }

 /* clcnut is a void
   to calculate the parameters of nutation in longitude and obliquity
      and value of the mean obliquity in radians on the moment tmjd.
      the arguments and coefficients
          of nutation theory are contained in the nutarg and coefnut .
      global parameter - secrad - for change from arcsecs to radians . */

 void clcnut ( double tmjd , double  *deltapsi , double *deltaeps )
 {
   int    i,k ;
   double r,dt ;
   double fundarg[6] ;
   double secrad ;
   /* 1980 iau theory of nutation  ( J. M. Wahr ) */
   int   nutarg[106][5] =   /* arguments */
             { { 1 ,    0 ,    0 ,    0 ,    0  },
               { 2 ,    0 ,    0 ,    0 ,    0  },
               { 1 ,   -2 ,    0 ,    2 ,    0  },
               { 0 ,    2 ,    0 ,   -2 ,    0  },
               { 2 ,   -2 ,    0 ,    2 ,    0  },
               { 0 ,    1 ,   -1 ,    0 ,   -1  },
               { 1 ,    0 ,   -2 ,    2 ,   -2  },
               { 1 ,    2 ,    0 ,   -2 ,    0  },
               { 2 ,    0 ,    0 ,    2 ,   -2  },
               { 0 ,    0 ,    1 ,    0 ,    0  },
               { 2 ,    0 ,    1 ,    2 ,   -2  },
               { 2 ,    0 ,   -1 ,    2 ,   -2  },
               { 1 ,    0 ,    0 ,    2 ,   -2  },
               { 0 ,    2 ,    0 ,    0 ,   -2  },
               { 0 ,    0 ,    0 ,    2 ,   -2  },
               { 0 ,    0 ,    2 ,    0 ,    0  },
               { 1 ,    0 ,    1 ,    0 ,    0  },
               { 2 ,    0 ,    2 ,    2 ,   -2  },
               { 1 ,    0 ,   -1 ,    0 ,    0  },
               { 1 ,   -2 ,    0 ,    0 ,    2  },
               { 1 ,    0 ,   -1 ,    2 ,   -2  },
               { 1 ,    2 ,    0 ,    0 ,   -2  },
               { 1 ,    0 ,    1 ,    2 ,   -2  },
               { 0 ,    1 ,    0 ,    0 ,   -1  },
               { 0 ,    2 ,    1 ,    0 ,   -2  },
               { 1 ,    0 ,    0 ,   -2 ,    2  },
               { 0 ,    0 ,    1 ,   -2 ,    2  },
               { 2 ,    0 ,    1 ,    0 ,    0  },
               { 1 ,   -1 ,    0 ,    0 ,    1  },
               { 0 ,    0 ,    1 ,    2 ,   -2  },
               { 2 ,    0 ,    0 ,    2 ,    0  },
               { 0 ,    1 ,    0 ,    0 ,    0  },
               { 1 ,    0 ,    0 ,    2 ,    0  },
               { 2 ,    1 ,    0 ,    2 ,    0  },
               { 0 ,    1 ,    0 ,    0 ,   -2  },
               { 2 ,   -1 ,    0 ,    2 ,    0  },
               { 0 ,    0 ,    0 ,    0 ,    2  },
               { 1 ,    1 ,    0 ,    0 ,    0  },
               { 1 ,   -1 ,    0 ,    0 ,    0  },
               { 2 ,   -1 ,    0 ,    2 ,    2  },
               { 1 ,    1 ,    0 ,    2 ,    0  },
               { 2 ,    0 ,    0 ,    2 ,    2  },
               { 0 ,    2 ,    0 ,    0 ,    0  },
               { 2 ,    1 ,    0 ,    2 ,   -2  },
               { 2 ,    2 ,    0 ,    2 ,    0  },
               { 0 ,    0 ,    0 ,    2 ,    0  },
               { 1 ,   -1 ,    0 ,    2 ,    0  },
               { 1 ,   -1 ,    0 ,    0 ,    2  },
               { 1 ,    1 ,    0 ,    0 ,   -2  },
               { 1 ,   -1 ,    0 ,    2 ,    2  },
               { 0 ,    1 ,    1 ,    0 ,   -2  },
               { 2 ,    0 ,    1 ,    2 ,    0  },
               { 2 ,    0 ,   -1 ,    2 ,    0  },
               { 2 ,    1 ,    0 ,    2 ,    2  },
               { 0 ,    1 ,    0 ,    0 ,    2  },
               { 2 ,    2 ,    0 ,    2 ,   -2  },
               { 1 ,    0 ,    0 ,    0 ,    2  },
               { 1 ,    0 ,    0 ,    2 ,    2  },
               { 1 ,    1 ,    0 ,    2 ,   -2  },
               { 1 ,    0 ,    0 ,    0 ,   -2  },
               { 0 ,    1 ,   -1 ,    0 ,    0  },
               { 1 ,    2 ,    0 ,    2 ,    0  },
               { 0 ,    0 ,    1 ,    0 ,   -2  },
               { 0 ,    1 ,    0 ,   -2 ,    0  },
               { 0 ,    0 ,    0 ,    0 ,    1  },
               { 0 ,    1 ,    1 ,    0 ,    0  },
               { 0 ,    1 ,    0 ,    2 ,    0  },
               { 2 ,    1 ,   -1 ,    2 ,    0  },
               { 2 ,   -1 ,   -1 ,    2 ,    2  },
               { 1 ,   -2 ,    0 ,    0 ,    0  },
               { 2 ,    3 ,    0 ,    2 ,    0  },
               { 2 ,    0 ,   -1 ,    2 ,    2  },
               { 2 ,    1 ,    1 ,    2 ,    0  },
               { 1 ,   -1 ,    0 ,    2 ,   -2  },
               { 1 ,    2 ,    0 ,    0 ,    0  },
               { 2 ,    1 ,    0 ,    0 ,    0  },
               { 0 ,    3 ,    0 ,    0 ,    0  },
               { 2 ,    0 ,    0 ,    2 ,    1  },
               { 2 ,   -1 ,    0 ,    0 ,    0  },
               { 0 ,    1 ,    0 ,    0 ,   -4  },
               { 2 ,   -2 ,    0 ,    2 ,    2  },
               { 2 ,   -1 ,    0 ,    2 ,    4  },
               { 0 ,    2 ,    0 ,    0 ,   -4  },
               { 2 ,    1 ,    1 ,    2 ,   -2  },
               { 1 ,    1 ,    0 ,    2 ,    2  },
               { 2 ,   -2 ,    0 ,    2 ,    4  },
               { 2 ,   -1 ,    0 ,    4 ,    0  },
               { 0 ,    1 ,   -1 ,    0 ,   -2  },
               { 1 ,    2 ,    0 ,    2 ,   -2  },
               { 2 ,    2 ,    0 ,    2 ,    2  },
               { 1 ,    1 ,    0 ,    0 ,    2  },
               { 2 ,    0 ,    0 ,    4 ,   -2  },
               { 2 ,    3 ,    0 ,    2 ,   -2  },
               { 0 ,    1 ,    0 ,    2 ,   -2  },
               { 1 ,    0 ,    1 ,    2 ,    0  },
               { 1 ,   -1 ,   -1 ,    0 ,    2  },
               { 1 ,    0 ,    0 ,   -2 ,    0  },
               { 2 ,    0 ,    0 ,    2 ,   -1  },
               { 0 ,    0 ,    1 ,    0 ,    2  },
               { 0 ,    1 ,    0 ,   -2 ,   -2  },
               { 1 ,    0 ,   -1 ,    2 ,    0  },
               { 1 ,    1 ,    1 ,    0 ,   -2  },
               { 0 ,    1 ,    0 ,   -2 ,    2  },
               { 0 ,    2 ,    0 ,    0 ,    2  },
               { 2 ,    0 ,    0 ,    2 ,    4  },
               { 0 ,    0 ,    1 ,    0 ,    1  } } ;
   /* 1980 iau theory of nutation  ( J. M. Wahr ) */
   float coefnut[106][4] =   /* amplitudes */
      {  { -17.199600 ,     -0.017420 ,      9.202500 ,      0.000890  },
         {  0.206200 ,      0.000020 ,     -0.089500 ,      0.000050  },
         {  0.004600 ,      0.000000 ,     -0.002400 ,      0.000000  },
         {  0.001100 ,      0.000000 ,      0.000000 ,      0.000000  },
         { -0.000300 ,      0.000000 ,      0.000100 ,      0.000000  },
         { -0.000300 ,      0.000000 ,      0.000000 ,      0.000000  },
         { -0.000200 ,      0.000000 ,      0.000100 ,      0.000000  },
         {  0.000100 ,      0.000000 ,      0.000000 ,      0.000000  },
         { -1.318700 ,     -0.000160 ,      0.573600 ,     -0.000310  },
         {  0.142600 ,     -0.000340 ,      0.005400 ,     -0.000010  },
         { -0.051700 ,      0.000120 ,      0.022400 ,     -0.000060  },
         {  0.021700 ,     -0.000050 ,     -0.009500 ,      0.000030  },
         {  0.012900 ,      0.000010 ,     -0.007000 ,      0.000000  },
         {  0.004800 ,      0.000000 ,      0.000100 ,      0.000000  },
         { -0.002200 ,      0.000000 ,      0.000000 ,      0.000000  },
         {  0.001700 ,     -0.000010 ,      0.000000 ,      0.000000  },
         { -0.001500 ,      0.000000 ,      0.000900 ,      0.000000  },
         { -0.001600 ,      0.000010 ,      0.000700 ,      0.000000  },
         { -0.001200 ,      0.000000 ,      0.000600 ,      0.000000  },
         { -0.000600 ,      0.000000 ,      0.000300 ,      0.000000  },
         { -0.000500 ,      0.000000 ,      0.000300 ,      0.000000  },
         {  0.000400 ,      0.000000 ,     -0.000200 ,      0.000000  },
         {  0.000400 ,      0.000000 ,     -0.000200 ,      0.000000  },
         { -0.000400 ,      0.000000 ,      0.000000 ,      0.000000  },
         {  0.000100 ,      0.000000 ,      0.000000 ,      0.000000  },
         {  0.000100 ,      0.000000 ,      0.000000 ,      0.000000  },
         { -0.000100 ,      0.000000 ,      0.000000 ,      0.000000  },
         {  0.000100 ,      0.000000 ,      0.000000 ,      0.000000  },
         {  0.000100 ,      0.000000 ,      0.000000 ,      0.000000  },
         { -0.000100 ,      0.000000 ,      0.000000 ,      0.000000  },
         { -0.227400 ,     -0.000020 ,      0.097700 ,     -0.000050  },
         {  0.071200 ,      0.000010 ,     -0.000700 ,      0.000000  },
         { -0.038600 ,     -0.000040 ,      0.020000 ,      0.000000  },
         { -0.030100 ,      0.000000 ,      0.012900 ,     -0.000010  },
         { -0.015800 ,      0.000000 ,     -0.000100 ,      0.000000  },
         {  0.012300 ,      0.000000 ,     -0.005300 ,      0.000000  },
         {  0.006300 ,      0.000000 ,     -0.000200 ,      0.000000  },
         {  0.006300 ,      0.000010 ,     -0.003300 ,      0.000000  },
         { -0.005800 ,     -0.000010 ,      0.003200 ,      0.000000  },
         { -0.005900 ,      0.000000 ,      0.002600 ,      0.000000  },
         { -0.005100 ,      0.000000 ,      0.002700 ,      0.000000  },
         { -0.003800 ,      0.000000 ,      0.001600 ,      0.000000  },
         {  0.002900 ,      0.000000 ,     -0.000100 ,      0.000000  },
         {  0.002900 ,      0.000000 ,     -0.001200 ,      0.000000  },
         { -0.003100 ,      0.000000 ,      0.001300 ,      0.000000  },
         {  0.002600 ,      0.000000 ,     -0.000100 ,      0.000000  },
         {  0.002100 ,      0.000000 ,     -0.001000 ,      0.000000  },
         {  0.001600 ,      0.000000 ,     -0.000800 ,      0.000000  },
         { -0.001300 ,      0.000000 ,      0.000700 ,      0.000000  },
         { -0.001000 ,      0.000000 ,      0.000500 ,      0.000000  },
         { -0.000700 ,      0.000000 ,      0.000000 ,      0.000000  },
         {  0.000700 ,      0.000000 ,     -0.000300 ,      0.000000  },
         { -0.000700 ,      0.000000 ,      0.000300 ,      0.000000  },
         { -0.000800 ,      0.000000 ,      0.000300 ,      0.000000  },
         {  0.000600 ,      0.000000 ,      0.000000 ,      0.000000  },
         {  0.000600 ,      0.000000 ,     -0.000300 ,      0.000000  },
         { -0.000600 ,      0.000000 ,      0.000300 ,      0.000000  },
         { -0.000700 ,      0.000000 ,      0.000300 ,      0.000000  },
         {  0.000600 ,      0.000000 ,     -0.000300 ,      0.000000  },
         { -0.000500 ,      0.000000 ,      0.000300 ,      0.000000  },
         {  0.000500 ,      0.000000 ,      0.000000 ,      0.000000  },
         { -0.000500 ,      0.000000 ,      0.000300 ,      0.000000  },
         { -0.000400 ,      0.000000 ,      0.000000 ,      0.000000  },
         {  0.000400 ,      0.000000 ,      0.000000 ,      0.000000  },
         { -0.000400 ,      0.000000 ,      0.000000 ,      0.000000  },
         { -0.000300 ,      0.000000 ,      0.000000 ,      0.000000  },
         {  0.000300 ,      0.000000 ,      0.000000 ,      0.000000  },
         { -0.000300 ,      0.000000 ,      0.000100 ,      0.000000  },
         { -0.000300 ,      0.000000 ,      0.000100 ,      0.000000  },
         { -0.000200 ,      0.000000 ,      0.000100 ,      0.000000  },
         { -0.000300 ,      0.000000 ,      0.000100 ,      0.000000  },
         { -0.000300 ,      0.000000 ,      0.000100 ,      0.000000  },
         {  0.000200 ,      0.000000 ,     -0.000100 ,      0.000000  },
         { -0.000200 ,      0.000000 ,      0.000100 ,      0.000000  },
         {  0.000200 ,      0.000000 ,     -0.000100 ,      0.000000  },
         { -0.000200 ,      0.000000 ,      0.000100 ,      0.000000  },
         {  0.000200 ,      0.000000 ,      0.000000 ,      0.000000  },
         {  0.000200 ,      0.000000 ,     -0.000100 ,      0.000000  },
         {  0.000100 ,      0.000000 ,     -0.000100 ,      0.000000  },
         { -0.000100 ,      0.000000 ,      0.000000 ,      0.000000  },
         {  0.000100 ,      0.000000 ,     -0.000100 ,      0.000000  },
         { -0.000200 ,      0.000000 ,      0.000100 ,      0.000000  },
         { -0.000100 ,      0.000000 ,      0.000000 ,      0.000000  },
         {  0.000100 ,      0.000000 ,     -0.000100 ,      0.000000  },
         { -0.000100 ,      0.000000 ,      0.000100 ,      0.000000  },
         { -0.000100 ,      0.000000 ,      0.000100 ,      0.000000  },
         {  0.000100 ,      0.000000 ,      0.000000 ,      0.000000  },
         {  0.000100 ,      0.000000 ,      0.000000 ,      0.000000  },
         {  0.000100 ,      0.000000 ,     -0.000100 ,      0.000000  },
         { -0.000100 ,      0.000000 ,      0.000000 ,      0.000000  },
         { -0.000100 ,      0.000000 ,      0.000000 ,      0.000000  },
         {  0.000100 ,      0.000000 ,      0.000000 ,      0.000000  },
         {  0.000100 ,      0.000000 ,      0.000000 ,      0.000000  },
         { -0.000100 ,      0.000000 ,      0.000000 ,      0.000000  },
         {  0.000100 ,      0.000000 ,      0.000000 ,      0.000000  },
         {  0.000100 ,      0.000000 ,      0.000000 ,      0.000000  },
         { -0.000100 ,      0.000000 ,      0.000000 ,      0.000000  },
         { -0.000100 ,      0.000000 ,      0.000000 ,      0.000000  },
         { -0.000100 ,      0.000000 ,      0.000000 ,      0.000000  },
         { -0.000100 ,      0.000000 ,      0.000000 ,      0.000000  },
         { -0.000100 ,      0.000000 ,      0.000000 ,      0.000000  },
         { -0.000100 ,      0.000000 ,      0.000000 ,      0.000000  },
         { -0.000100 ,      0.000000 ,      0.000000 ,      0.000000  },
         {  0.000100 ,      0.000000 ,      0.000000 ,      0.000000  },
         { -0.000100 ,      0.000000 ,      0.000000 ,      0.000000  },
         {  0.000100 ,      0.000000 ,      0.000000 ,      0.000000  } } ;

   secrad = M_PI/(180.0*3600.0);
   clcfundarg(tmjd,fundarg);
   dt = (tmjd-51544.5e0)/36525.0e0;
   *deltapsi = 0.0;
   *deltaeps = 0.0;
   for (  k = 0 ;  k <= 105 ; k++ )
     {
       r = nutarg[k][0]*(fundarg[1]-fundarg[4]);
       for (  i = 1 ;  i <= 4 ; i++ )
         r = r + nutarg[k][i] * fundarg[i+1];
       *deltapsi = *deltapsi+(coefnut[k][0]
                       +dt*coefnut[k][1])*sin(r);
       *deltaeps = *deltaeps+(coefnut[k][2]
                       +dt*coefnut[k][3])*cos(r);
     }
   *deltapsi = secrad*(*deltapsi); /* in radian */
   *deltaeps = secrad*(*deltaeps);
 }

