
/* unit unjob133 to integrate orbit */

#include <conio.h>
#include <stdio.h>
#include <io.h>

#include "unjob133.h"

#include "unephjpl.h"   /* for planets ephemeris */
#include "unforkep.h"   /* for Kepler procedures */
#include "uneverim.h"   /* method to integrate */
#include "unfordat.h"   /* transmjdtodate procedure */

void forjob133 ()
{
  double start,finish ;      /* moment in mjd */
  double pose[4],ange[4] ;   /* kepler elements initial */
  double posc[4],angc[4] ;   /* kepler elements current */
  double vecr[4],vecv[4] ;   /* rectangular position velosity */

  double  dstep = 1.0 ; /* step in day for integration */

                  /* variables for integration step by step */
  double  ta ;    /* moment in mjd for integartion */
  double  xa[4] ; /* position in AU for integartion */
  double  va[4] ; /* velosity in AU/day for integartion */
  double  tb ;    /* moment in mjd for integartion */
  double  xb[4] ; /* position in AU for integartion */
  double  vb[4] ; /* velosity in AU/day for integartion */

  double  tc ;    /* moment in mjd to print result */
  double  dc ;    /* moment in day from the current interval start */
  double  xc[4] ; /* current position in AU */
  double  vc[4] ; /* current velosity in AU/day */

  int nd,nm,ny ;  /* day month year */
  int kh,km ;     /* hour minute */
  double sec ;    /* second */

  printf("\n");
  printf("to integrate orbit\n");

  start  = 53000.0 ;         /* epoch of the elements in mjd */

  finish = 51545.0 + 10.0*365.25 ;   /* moment for the result */

                          /* ecliptic heliocentric 1994 GK */

  pose[1] =  1.98661100;  /* semi-major axis AU */
  pose[2] =  0.61091653;  /* eccentricity */
  pose[3] =  5.714540;    /* inclination in degree */

  ange[1] = 15.205993;    /* the ascending node in degree */
  ange[2] =111.733859;    /* argument of perigei in degree */
  ange[3] =186.566521;    /* mean anomaly in degree */


  /* these two voids is necessary to work */

  toobtainfmus () ;  /* unephjpl for global factormuz[..] */

  toopencoefile () ; /* unephjpl coefile global */

  /* from Kepler ecliptic elements
       to descart equator position velosity in AU AU/day */

  fromkeplertocart(start,pose,ange,start,vecr,vecv); /* from unforkep */

  printf(" \n");
  printf("   input\n");
  printf("   minor body 1994 GK\n");
  printf("   initial elements ecliptic heliocentric\n");
  printf("   %19.8f  epoch in mjd         \n",start);
  printf("   %19.8f  a semimajor axis   AU\n",pose[1]);
  printf("   %19.8f  e eccentricity       \n",pose[2]);
  printf("   %19.6f  i inclination degree \n",pose[3]);
  printf("   %19.6f  O ascending node deg.\n",ange[1]);
  printf("   %19.6f  o arg.perihelium deg.\n",ange[2]);
  printf("   %19.6f  M mean anomaly degree\n",ange[3]);

  printf(" \n");
  printf("   initial condition vector\n");
  printf("   coordinate and velosity equator heliocentric\n");
  printf("   %19.8f  moment in mjd        \n",start);
  printf("   %19.9f  x  coordinate   in AU\n",vecr[1]);
  printf("   %19.9f  y  coordinate   in AU\n",vecr[2]);
  printf("   %19.9f  z  coordinate   in AU\n",vecr[3]);
  printf("   %19.12f  vx velosity in AU/day\n",vecv[1]);
  printf("   %19.12f  vy velosity in AU/day\n",vecv[2]);
  printf("   %19.12f  vz velosity in AU/day\n",vecv[3]);

  /* initial conditions to integrate */

  ta = start ;     /* date in mjd */

  xa[1] = vecr[1]; /* x AU */
  xa[2] = vecr[2]; /* y AU */
  xa[3] = vecr[3]; /* z AU */

  va[1] = vecv[1]; /* v_x AU/day */
  va[2] = vecv[2]; /* v_y AU/day */
  va[3] = vecv[3]; /* v_z AU/day */

  tb = start ;     /* date in mjd */

  xb[1] = vecr[1]; /* x AU */
  xb[2] = vecr[2]; /* y AU */
  xb[3] = vecr[3]; /* z AU */

  vb[1] = vecv[1]; /* v_x AU/day */
  vb[2] = vecv[2]; /* v_y AU/day */
  vb[3] = vecv[3]; /* v_z AU/day */

  tc = 51545.0 ; /* current moment in mjd to print */

  if  ( tc < ta )        /* initial moment to print is less than */
    {                    /* initial moment of initial  condition */
      everinit () ;      /* uneverim it is necessary to start integration */
      evercoefnullo () ; /* uneverim it is necessary to start integration */

      while  ( tc < ta )  /* backward with negative step */
        posvelevergrator(-dstep,&ta,xa,va,&tb,xb,vb); /* uneverim */
      /* to new initial condition */
      ta = tb;       /* finish of interval */
      xa[1] = xb[1]; /* x */
      xa[2] = xb[2]; /* y */
      xa[3] = xb[3]; /* z */
      va[1] = vb[1]; /* v_x */
      va[2] = vb[2]; /* v_y */
      va[3] = vb[3]; /* v_z */
    }
                     /* initial condition for integration are  finded */
  everinit () ;      /* uneverim it is necessary to start integration */
  evercoefnullo () ; /* uneverim it is necessary to start integration */
                     /* uneverim one step to change interval  [ta,tb] */
  posvelevergrator(dstep,&ta,xa,va,&tb,xb,vb); /* uneverim */

  do                  /* to integrate and to print result */
  {                   /* for known moment tc in mod.julian day */
    while  ( tb < tc )   /* moment print out interval integration */
      posvelevergrator(dstep,&ta,xa,va,&tb,xb,vb); /* uneverim */

    if  ( ( ta <= tc ) &&  ( tc <= tb ) ) /* inner of interval */
      {
        dc = tc-ta ; /* difference with start moment in interval */
                     /* heliocentric equator descart position velosity */
        clcvectfrompolinom(dc,xa,va,xc,vc); /* uneverim current */
                     /* heliocentric ecliptic Kepler elements */
        fromcarttokepler(tc,xc,vc,tc,posc,angc); /* unforkep */

        transmjdtodate(tc,&nd,&nm,&ny,&kh,&km,&sec); /* proc form unfordat */

        printf("  %4i%3i%3i%4i%13.6f%13.7f%11.5f%12.5f%12.5f\n",
                ny,nm,nd,kh,posc[1],posc[2],posc[3],angc[1],angc[2]);

      }
    tc = tc+10.0; /* the next step in day to print */

  }
  while  ( tc < finish ) ; /* before finish date */


  close(coefile); /* global unephjpl */
}
