
/* unit unjob134 to integrate orbit with minimum range */

#include <math.h>
#include <conio.h>
#include <stdio.h>
#include <io.h>

#include "unjob134.h"

#include "unephjpl.h"   /* for planets ephemeris */
#include "unforkep.h"   /* for Kepler procedures */
#include "uneverim.h"   /* method to integrate */
#include "unfordat.h"   /* transmjdtodate procedure */

void forjob134 ()
{
  double start,finish ;      /* moment in mjd */
  double pose[4],ange[4] ;   /* kepler elements initial */
  double posc[4],angc[4] ;   /* kepler elements current */
  double vecr[4],vecv[4] ;   /* rectangular position velosity */

  double  dstep = 1.0 ; /* step in day for integration */

                  /* variables for integration step by step */
  double  ta ;    /* moment in mjd for integration */
  double  xa[4] ; /* position in AU for integration */
  double  va[4] ; /* velosity in AU/day for integration */
  double  tb ;    /* moment in mjd for integration */
  double  xb[4] ; /* position in AU for integration */
  double  vb[4] ; /* velosity in AU/day for integration */

  double  tc ;    /* moment in mjd to print result */
  double  dc ;    /* moment in day from the current interval start */
  double  xc[4] ; /* current position in AU */
  double  vc[4] ; /* current velosity in AU/day */

  double  xe[4] ; /* current position of the Earth in AU */
  double  ve[4] ; /* current velosity of the Earth in AU */

  double  dr[4] ; /* vector of differences in AU */
  double  range ; /* range between the Earth and the minor body */

  int nd,nm,ny ;  /* day month year */
  int kh,km ;     /* hour minute */
  double sec ;    /* second */

  printf("\n");
  printf("to integrate orbit with minimum range from the Earth\n");

  start  = 53000.0 ;         /* epoch of the elements in mjd */

  finish = 51545.0 + 10.0*365.25 ;   /* moment for the result */

                          /* ecliptic heliocentric 1994 GK */

  pose[1] =  1.98661100;  /* semi-major axis AU */
  pose[2] =  0.61091653;  /* eccentricity */
  pose[3] =  5.714540;    /* inclination in degree */

  ange[1] = 15.205993;    /* the ascending node in degree */
  ange[2] =111.733859;    /* argument of perigei in degree */
  ange[3] =186.566521;    /* mean anomaly in degree */

  /* these two voids is necessary to work */

  toobtainfmus () ;  /* unephjpl for global factormuz[..] */

  toopencoefile () ; /* unephjpl coefile global */

  /* from Kepler ecliptic elements
       to descart equator position velosity in AU AU/day */

  fromkeplertocart(start,pose,ange,start,vecr,vecv); /* from unforkep */

  printf(" \n");
  printf("   input\n");
  printf("   minor body 1994 GK\n");
  printf("   initial elements ecliptic heliocentric\n");
  printf("   %19.8f  epoch in mjd         \n",start);
  printf("   %19.8f  a semimajor axis   AU\n",pose[1]);
  printf("   %19.8f  e eccentricity       \n",pose[2]);
  printf("   %19.6f  i inclination degree \n",pose[3]);
  printf("   %19.6f  O ascending node deg.\n",ange[1]);
  printf("   %19.6f  o arg.perihelium deg.\n",ange[2]);
  printf("   %19.6f  M mean anomaly degree\n",ange[3]);

  printf(" \n");
  printf("   initial condition vector\n");
  printf("   coordinate and velosity equator heliocentric\n");
  printf("   %19.8f  moment in mjd        \n",start);
  printf("   %19.9f  x  coordinate   in AU\n",vecr[1]);
  printf("   %19.9f  y  coordinate   in AU\n",vecr[2]);
  printf("   %19.9f  z  coordinate   in AU\n",vecr[3]);
  printf("   %19.12f  vx velosity in AU/day\n",vecv[1]);
  printf("   %19.12f  vy velosity in AU/day\n",vecv[2]);
  printf("   %19.12f  vz velosity in AU/day\n",vecv[3]);

  /* initial conditions to integrate */

  ta = start ;     /* date in mjd */

  xa[1] = vecr[1]; /* x AU */
  xa[2] = vecr[2]; /* y AU */
  xa[3] = vecr[3]; /* z AU */

  va[1] = vecv[1]; /* v_x AU/day */
  va[2] = vecv[2]; /* v_y AU/day */
  va[3] = vecv[3]; /* v_z AU/day */

  tb = start ;     /* date in mjd */

  xb[1] = vecr[1]; /* x AU */
  xb[2] = vecr[2]; /* y AU */
  xb[3] = vecr[3]; /* z AU */

  vb[1] = vecv[1]; /* v_x AU/day */
  vb[2] = vecv[2]; /* v_y AU/day */
  vb[3] = vecv[3]; /* v_z AU/day */

  everinit () ;      /* uneverim it is necessary to start integration */
  evercoefnullo () ; /* uneverim it is necessary to start integration */
                     /* uneverim one step to change interval  [ta,tb] */
  posvelevergrator(dstep,&ta,xa,va,&tb,xb,vb); /* uneverim */

  tc = start ; /* current moment in mjd to print */

  do                  /* to integrate and to print result */
  {                   /* for known moment tc in mod.julian day */
    if  ( tb < tc )   /* moment print out interval integration */
      posvelevergrator(dstep,&ta,xa,va,&tb,xb,vb); /* uneverim */

    if  ( ( ta <= tc ) &&  ( tc <= tb ) ) /* inner of interval */
      {
        dc = tc-ta ; /* difference with start moment in interval */
                     /* heliocentric equator descart position velosity */
        clcvectfrompolinom(dc,xa,va,xc,vc); /* uneverim minor body */

        earths(tc,xe,ve); /* unephjpl for the Earth refer to the Sun */

        dr[1] = xc[1]-xe[1]; /* x AU minor body refer to the Earth */
        dr[2] = xc[2]-xe[2]; /* y AU minor body refer to the Earth */
        dr[3] = xc[3]-xe[3]; /* z AU minor body refer to the Earth */

        range = sqrt(dr[1]*dr[1]+dr[2]*dr[2]+dr[3]*dr[3]); /* distance AU */

        if  ( range < 0.05 )  /* distance in AU to compare */
          {
            transmjdtodate(tc,&nd,&nm,&ny,&kh,&km,&sec); /* unfordat */

                     /* heliocentric ecliptic Kepler elements */
            fromcarttokepler(tc,xc,vc,tc,posc,angc); /* unforkep */

            printf("  %4i%3i%3i%4i%13.6f%13.7f%11.5f%12.5f%12.1f\n",
                   ny,nm,nd,kh,posc[1],posc[2],posc[3],
                   range,1.49597870691e8*range);
          }

      }
    tc = tc+0.5; /* the next step in day */

  }
  while  ( tc < finish ) ; /* before finish date */


  close(coefile); /* global unephjpl */
}
