/*
 99942  may be the number
  Apophis    may be the name
       2454200.50000  epoch of the elements
         0.922261440  semimajor axis in AU
         0.191059390  eccentricity
           3.3313130  angle of inclination in degree
         204.4592500  the ascending node in degree
         126.3854880  the argument of perihelium
         307.3630340  mean anomaly in degree
               19.20  parameter H
                0.15  parameter G
 99942  may be the number
  Apophis    may be the name
       2453442.50000  epoch of the elements
         0.922377951  semimajor axis in AU           0.000000000  error
         0.191054854  eccentricity
           3.3309207  angle of inclination in degree
         204.4721548  the ascending node in degree
         126.3839707  the argument of perihelium
         183.9470916  mean anomaly in degree         0.000000000  error
               19.70  parameter H
                0.25  parameter G
*/
/*
           2024     number in file
                    may be the number of the asteroid
  2004 HW           may be the name of the asteroid
 2454200.50         elements epoch in julian day
    2.688533970     semimajor axis in astronomical unit
    0.636971710     eccentricity
    0.8329270       inclination in degree
  220.4296490       longitude of ascension node in degree
   62.2992410       argument of perihelion in degree
  234.6149310       mean anomaly in degree
   17.130           parametr H
    0.150           parametr G
/*
/* unit unjob135 to integrate orbit with minimum range and magnitude */

#include <math.h>
#include <conio.h>
#include <stdio.h>
#include <io.h>

#include "unjob135.h"

#include "unephjpl.h"   /* for planets ephemeris */
#include "unforkep.h"   /* for Kepler procedures */
#include "uneverim.h"   /* method to integrate */
#include "unfordat.h"   /* transmjdtodate procedure */
#include "unastvam.h"   /* togetmau function */
#include "unfortim.h"   /* transutctotdt transtdttotdb functions */

void forjob135 ()
{
  double elepo ;             /* epoch of the elements in mjd */
  double pose[4],ange[4] ;   /* kepler elements initial */
  double posc[4],angc[4] ;   /* kepler elements current */
  double vecr[4],vecv[4] ;   /* rectangular position velosity */

  double valh,valg ;         /* minor body abs.magnitude H parameter G */

  double start,finish ;      /* moments in mjd for results */
  double  dstep ;            /* step in day for integration */
  double  stepr ;            /* step in day to print result */

                  /* variables for integration step by step */
  double  ta ;    /* moment in mjd for integration */
  double  xa[4] ; /* position in AU for integration */
  double  va[4] ; /* velosity in AU/day for integration */
  double  tb ;    /* moment in mjd for integration */
  double  xb[4] ; /* position in AU for integration */
  double  vb[4] ; /* velosity in AU/day for integration */

  double  tc ;    /* moment in mjd to print result */
  double  dc ;    /* moment in day from the current interval start */
  double  xc[4] ; /* current position in AU refer to the Sun */
  double  vc[4] ; /* current velosity in AU/day */

  double  xe[4] ; /* current position of the Earth in AU */
  double  ve[4] ; /* current velosity of the Earth in AU */

  double  dr[4] ; /* vector of differences in AU refer to the Earth */

/* here new variales */

  double  tdt ;   /* terrestrial time tt or tdt */
  double  dta ;   /* Delta tt = TDT-UTC */
  double  tdb ;   /* barycentric time tdb */

/* here new variales */

  double  range ; /* range between the Earth and the minor body */
  double  vam ;   /* magnitude */

  int nd,nm,ny ;  /* day month year */
  int kh,km ;     /* hour minute */
  double sec ;    /* second */

  int intofile ;  /* for function open */

  printf("\n");
  printf("to integrate orbit with minimum range and magnitude\n");

  dstep = 1.0; /* step for integration in day */

  stepr = 1440.0/1440.0;  /* step in day to print result */

  start  = 51545.0 + 8.0*365.25 ;    /* moment for the result */
  finish = 51545.0 + 38.0*365.25 ;   /* moment for the result */

  /* initial conditions minor body Apophis */

  elepo  = 53442.0 ;      /* epoch of the elements in mjd */

                          /* ecliptic heliocentric Apophis */

  pose[1] =  0.922377951;  /* semi-major axis AU */
  pose[2] =  0.191054854;  /* eccentricity */
  pose[3] =    3.3309207;  /* inclination in degree */

  ange[1] =  204.4721548;    /* the ascending node in degree */
  ange[2] =  126.3839707;    /* argument of perigei in degree */
  ange[3] =  183.9470916;    /* mean anomaly in degree */

  valh = 19.70; /* absolute magnitude H */
  valg =  0.25; /* incline parameter G */

  /* initial conditions minor body Apophis */

  /* these two voids is necessary to work */

  toobtainfmus () ;  /* unephjpl for global factormuz[..] */

  intofile = toopencoefile () ; /* unephjpl coefile global */

  printf(" \n");
  printf(" %5i   a try to open file\n",intofile);

  /* from Kepler ecliptic elements
       to descart equator position velosity in AU AU/day */

  fromkeplertocart(elepo,pose,ange,elepo,vecr,vecv); /* from unforkep */

  printf(" \n");
  printf("   input\n");
  printf("   minor body 99942 Apophis\n");
  printf("   initial elements ecliptic heliocentric\n");
  printf("   %19.8f  epoch in mjd         \n",elepo);
  printf("   %19.8f  a semimajor axis   AU\n",pose[1]);
  printf("   %19.8f  e eccentricity       \n",pose[2]);
  printf("   %19.6f  i inclination degree \n",pose[3]);
  printf("   %19.6f  O ascending node deg.\n",ange[1]);
  printf("   %19.6f  o arg.perihelium deg.\n",ange[2]);
  printf("   %19.6f  M mean anomaly degree\n",ange[3]);

  printf(" \n");
  printf("   initial condition vector\n");
  printf("   coordinate and velosity equator heliocentric\n");
  printf("   %19.8f  moment in mjd        \n",elepo);
  printf("   %19.9f  x  coordinate   in AU\n",vecr[1]);
  printf("   %19.9f  y  coordinate   in AU\n",vecr[2]);
  printf("   %19.9f  z  coordinate   in AU\n",vecr[3]);
  printf("   %19.12f  vx velosity in AU/day\n",vecv[1]);
  printf("   %19.12f  vy velosity in AU/day\n",vecv[2]);
  printf("   %19.12f  vz velosity in AU/day\n",vecv[3]);
  printf(" \n");

  /* initial conditions to integrate */

  ta = elepo ;     /* date in mjd */

  xa[1] = vecr[1]; /* x AU */
  xa[2] = vecr[2]; /* y AU */
  xa[3] = vecr[3]; /* z AU */

  va[1] = vecv[1]; /* v_x AU/day */
  va[2] = vecv[2]; /* v_y AU/day */
  va[3] = vecv[3]; /* v_z AU/day */

  tb = elepo ;     /* date in mjd */

  xb[1] = vecr[1]; /* x AU */
  xb[2] = vecr[2]; /* y AU */
  xb[3] = vecr[3]; /* z AU */

  vb[1] = vecv[1]; /* v_x AU/day */
  vb[2] = vecv[2]; /* v_y AU/day */
  vb[3] = vecv[3]; /* v_z AU/day */

  tc = start; /* current moment to start point for result */

  if  ( tc < ta )        /* initial moment to print is less than */
    {                    /* initial moment of initial  condition */
      everinit () ;      /* uneverim it is necessary to start integration */
      evercoefnullo () ; /* uneverim it is necessary to start integration */

      while  ( tc < ta )  /* backward with negative step */
        posvelevergrator(-dstep,&ta,xa,va,&tb,xb,vb); /* uneverim */
      /* to new initial condition */
      ta = tb;       /* finish of interval */
      xa[1] = xb[1]; /* x */
      xa[2] = xb[2]; /* y */
      xa[3] = xb[3]; /* z */
      va[1] = vb[1]; /* v_x */
      va[2] = vb[2]; /* v_y */
      va[3] = vb[3]; /* v_z */
    }

  dstep = 1.0; /* step for integration in day */

  everinit () ;      /* uneverim it is necessary to start integration */
  evercoefnullo () ; /* uneverim it is necessary to start integration */
                     /* uneverim one step to change interval  [ta,tb] */
  posvelevergrator(dstep,&ta,xa,va,&tb,xb,vb); /* uneverim */

  do                   /* to integrate and to print result */
  {                    /* for known moment tc in mod.julian day */
                       /* current moment tc equal start moment */
    tc = tc+stepr;     /* one step to print result */

    while  ( tc < ta )   /* to compare with interval */
      tc = tc+stepr;     /* one step to print result */

    while  ( tb < ( tc-1.0e-8 ) ) /* moment print out interval integration */
      posvelevergratos(&dstep,&ta,xa,va,&tb,xb,vb); /* uneverim to time tc */

    if  ( ( ta <= tc ) &&  ( tc <= tb ) ) /* inner of interval */
      {
        dc = tc-ta ; /* difference with start moment in interval */
                     /* heliocentric equator descart position velosity */
        clcvectfrompolinom(dc,xa,va,xc,vc); /* uneverim minor body */

/* here start  new operators */

        transutctotdt(tc,&tdt,&dta); /* unit unfortim terrestrial time tt */
        transtdttotdb(tdt,&tdb) ;    /* barycentric time tdb teph for ephemeris */
                           /* for the time dynamic barycentric TDB */
        earths(tdb,xe,ve); /* unephjpl for the Earth refer to the Sun */

/* here finish new operators */

        dr[1] = xc[1]-xe[1]; /* x AU minor body refer to the Earth */
        dr[2] = xc[2]-xe[2]; /* y AU minor body refer to the Earth */
        dr[3] = xc[3]-xe[3]; /* z AU minor body refer to the Earth */

        range = sqrt(dr[1]*dr[1]+dr[2]*dr[2]+dr[3]*dr[3]); /* distance AU */

        if  ( range < 15.0 )  /* distance in AU to compare */
          {
            transmjdtodate((tc+1.0e-7),&nd,&nm,&ny,&kh,&km,&sec); /* unfordat */

                     /* heliocentric ecliptic Kepler elements */
            fromcarttokepler(tc,xc,vc,tc,posc,angc); /* unforkep */

                     /* magnitude of the minor body */
            vam = togetmau(valh,valg,xc,dr); /* unastvam */

            printf("%4i%3i%3i%3i%3i%10.6f%10.7f%9.5f%9.5f%12.1f%5.1f%5.2f\n",
                   ny,nm,nd,kh,km,posc[1],posc[2],posc[3],
                   range,1.49597870691e8*range,vam,dstep);

          }

      }
  }
  while  ( tc < finish ) ; /* before finish date */


  close(coefile); /* global unephjpl */
}
