
/* unit unrandas
   to integrate orbit with random errors
   to obtain error ellipse on known moment */

#include <math.h>
#include <conio.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>

#include "unrandas.h"

#include "unephjpl.h"   /* for planets ephemeris */
#include "unforfun.h"   /* for function sqr */
#include "unforkep.h"   /* for Kepler procedures */
#include "uneverim.h"   /* method to integrate */
#include "unfordat.h"   /* transmjdtodate procedure */
#include "unfortim.h"   /* transutctotdt transtdttotdb functions */

void errellipse ( int numvar ,     /* number of random variants */
                  double elepo ,   /* epoch of the elements */
                  double pose[4] , /* positional elements */
                  double ange[4] , /* angle elements */
                  double spos[4] , /* sigma positional elements */
                  double sang[4] , /* sigma for angle elements */
                  double finish ,  /* moment to obtain error ellipse */
                  double difstart[4] , /* result error ellipse start */
                  double difinish[4] ) /* result error ellipse finish */
{
  int nvar ; /* count for variants */

  double rece[4],vece[4] ;   /* start  position velosity main orbit */
  double recf[4],vecf[4] ;   /* finish position velosity main orbit */
  double posc[4],angc[4] ;   /* kepler elements current */
  double vecr[4],vecv[4] ;   /* rectangular position velosity */

  double orbs[4][4] ; /* start  orbital matrix */
  double orbf[4][4] ; /* finish orbital matrix */

  double  dstep ; /* step in day for integration */
                  /* variables for integration step by step */
  double  ta ;    /* moment in mjd for integration */
  double  xa[4] ; /* position in AU for integration */
  double  va[4] ; /* velosity in AU/day for integration */
  double  tb ;    /* moment in mjd for integration */
  double  xb[4] ; /* position in AU for integration */
  double  vb[4] ; /* velosity in AU/day for integration */

  double  da[4] ; /* result error for start  ellipse */
  double  db[4] ; /* result error for finish ellipse */
  double  dd ;    /* temporary difference */

  double  oa[4] ; /* current difference equator */
  double  ob[4] ; /* current difference orbital */

/* initial nullo for error ellipse */

  da[1] = 0.0;
  da[2] = 0.0;
  da[3] = 0.0;

  db[1] = 0.0;
  db[2] = 0.0;
  db[3] = 0.0;

/* to start process */

  randomize(); /* stdlib.h time.h  for random values */

  dstep = 1.0; /* step for integration in day */

  /* these two voids is necessary to work */

  toobtainfmus () ;  /* unephjpl for global factormuz[..] */

  toopencoefile () ; /* unephjpl coefile global binary with ephemeris */

  nvar=0; /* nullo variant the main orbit */

  /* for the main orbit  from Kepler ecliptic elements
     to descart equator position velosity in AU AU/day */

  fromkeplertocart(elepo,pose,ange,elepo,rece,vece); /* from unforkep */

  toobtainorbitalmatr(rece,vece,orbs); /* start orbital matrix */

  do  /* numvar+1 variant with the main orbit */
  {
    if  ( nvar == 0 )  /* for the main orbit */
      {               /* without random error */
        posc[1]=pose[1]; /* without random value */
        posc[2]=pose[2]; /* without random value */
        posc[3]=pose[3]; /* without random value */

        angc[1]=ange[1]; /* without random value */
        angc[2]=ange[2]; /* without random value */
        angc[3]=ange[3]; /* without random value */
      }
    else              /* for the numvar random orbits */
      {
        posc[1]=pose[1]+gaussrandomvalue(spos[1]); /* with random value */
        posc[2]=pose[2]+gaussrandomvalue(spos[2]); /* with random value */
        posc[3]=pose[3]+gaussrandomvalue(spos[3]); /* with random value */

        angc[1]=ange[1]+gaussrandomvalue(sang[1]); /* with random value */
        angc[2]=ange[2]+gaussrandomvalue(sang[2]); /* with random value */
        angc[3]=ange[3]+gaussrandomvalue(sang[3]); /* with random value */
    }

    /* from Kepler ecliptic elements
         to descart equator position velosity in AU AU/day */

    fromkeplertocart(elepo,posc,angc,elepo,vecr,vecv); /* from unforkep */

    if  ( nvar > 0 )
    {
      oa[1] = vecr[1]-rece[1] ;
      oa[2] = vecr[2]-rece[2] ;
      oa[3] = vecr[3]-rece[3] ;
      multmatrvec(orbs,oa,ob);  /* unforfun to orbital system */
      dd = fabs( ob[1] ) ;      /* temporary difference */
      if  ( dd > da[1] ) da[1] = dd ;  /* for start error */
      dd = fabs( ob[2] ) ;      /* temporary difference */
      if  ( dd > da[2] ) da[2] = dd ;
      dd = fabs( ob[3] ) ;      /* temporary difference */
      if  ( dd > da[3] ) da[3] = dd ;
    }

    /* initial conditions to integrate */

    ta = elepo ;     /* date in mjd */

    xa[1] = vecr[1]; /* x AU */
    xa[2] = vecr[2]; /* y AU */
    xa[3] = vecr[3]; /* z AU */

    va[1] = vecv[1]; /* v_x AU/day */
    va[2] = vecv[2]; /* v_y AU/day */
    va[3] = vecv[3]; /* v_z AU/day */

    tb = elepo ;     /* date in mjd */

    xb[1] = vecr[1]; /* x AU */
    xb[2] = vecr[2]; /* y AU */
    xb[3] = vecr[3]; /* z AU */

    vb[1] = vecv[1]; /* v_x AU/day */
    vb[2] = vecv[2]; /* v_y AU/day */
    vb[3] = vecv[3]; /* v_z AU/day */

    ta = tb;       /* to start integration */

    xa[1] = xb[1]; /* x */
    xa[2] = xb[2]; /* y */
    xa[3] = xb[3]; /* z */

    va[1] = vb[1]; /* v_x */
    va[2] = vb[2]; /* v_y */
    va[3] = vb[3]; /* v_z */

    everinit () ;      /* uneverim it is necessary to start integration */
    evercoefnullo () ; /* uneverim it is necessary to start integration */

                     /* uneverim step by step */
    while  ( tb < ( finish-1.0e-8 ) ) /* to finish moment of integration */
      posvelevergrator(dstep,&ta,xa,va,&tb,xb,vb); /* uneverim */

    if  ( nvar == 0 )
    {
      recf[1] = xb[1]; /* finish value main orbit */
      recf[2] = xb[2]; /* finish value main orbit */
      recf[3] = xb[3]; /* finish value main orbit */
      vecf[1] = vb[1]; /* finish value main orbit */
      vecf[2] = vb[2]; /* finish value main orbit */
      vecf[3] = vb[3]; /* finish value main orbit */
      toobtainorbitalmatr(recf,vecf,orbf); /* finish orbital matrix */
    }
    else
    {
      oa[1] = xb[1]-recf[1] ;
      oa[2] = xb[2]-recf[2] ;
      oa[3] = xb[3]-recf[3] ;
      multmatrvec(orbf,oa,ob);  /* unforfun to orbital system */
      dd = fabs( ob[1] ) ;      /* temporary difference */
      if  ( dd > db[1] ) db[1] = dd ;   /* for finish error ellipse */
      dd = fabs( ob[2] ) ;      /* temporary difference */
      if  ( dd > db[2] ) db[2] = dd ;   /* for finish error ellipse */
      dd = fabs( ob[3] ) ;      /* temporary difference */
      if  ( dd > db[3] ) db[3] = dd ;   /* for finish error ellipse */
    }

    nvar=nvar+1; /* the next variant */
  }
  while  ( nvar < (numvar+1) ) ; /* for all variants */

  difstart[1] = da[1]; /* start  error ellipse */
  difstart[2] = da[2]; /* start  error ellipse */
  difstart[3] = da[3]; /* start  error ellipse */

  difinish[1] = db[1]; /* finish error ellipse */
  difinish[2] = db[2]; /* finish error ellipse */
  difinish[3] = db[3]; /* finish error ellipse */

}

 /* from the book
    Polyakov D.V., Kruglov I.Yu.
    about Turbo Pascal version 5.5
    Moscov  MAI  1992 year */

 double gaussrandomvalue ( double sigma )
 {
   double a,b,r,s;

   do
   {
     a=2.0*(random(10000)/10000.0)-1.0;
     b=2.0*(random(10000)/10000.0)-1.0;
     r=sqr(a)+sqr(b);
   }
   while ( r > 1.0 ) ;

   s=sqrt(-2.0*log(r)/r);

   return (sigma*a*s);  /* random gauss value */
 }


/* orbmatr
     orbital matrix to change
     from equator system
     to orbital system refer to the minor planet
     axis OX direct as transversal from the asteroid
     axis OZ direct along radius-vector
     R_x(90)*R_z(90)*R_z(u)*R_x(i)*R_z(Omega)
     where simple orbit parameters
     u is argument of latitude
     i is angle of inclination
     Omega is longitude of ascending node */

 void toobtainorbitalmatr ( double r[4] ,
                            double v[4] ,
                            double orbmatr[4][4] )  /* result matrix */
 {
   int  i ;
   double c[4],d[4] ;
   double p,q,si,ci ;
   double cucn,cusn ;
   double su,cu,sn,cn ;

   /* integrals of the angular momentum */
   c[1] = r[2]*v[3]-v[2]*r[3]; /* +sin(omega)*sin(i)*c */
   c[2] = r[3]*v[1]-v[3]*r[1]; /* -cos(omega)*sin(i)*c */
   c[3] = r[1]*v[2]-v[1]*r[2]; /* +cos(i)*c */
   q = sqrt(sqr(r[1])+sqr(r[2])+sqr(r[3])); /* the modul */
   p = sqrt(sqr(c[1])+sqr(c[2])+sqr(c[3])); /* the modul */
   for (  i = 1 ;  i <= 3 ; i++ )
     { /* to unit vector */
       d[i] = r[i]/q;
       c[i] = c[i]/p;
     }
   ci = c[3]; /* cos(inclination) */
   si = sqrt(sqr(c[1])+sqr(c[2])); /* sin(inclination) */
   sn = +c[1]/si; /* sin(the ascending node) */
   cn = -c[2]/si; /* cos(the ascending node) */
   su = +d[3]/si; /* sin(u) or sin(argument of the latitude) */
   cucn = d[1]+su*sn*ci; /* cos(u)*cos(omega) */
   cusn = d[2]-su*cn*ci; /* cos(u)*sin(omega) */
   if  ( fabs(cn) > 0.1 )  cu = cucn/cn ; /* cos(u) */
   else  cu = cusn/sn;
   for (  i = 1 ;  i <= 3 ; i++ )
     {
       orbmatr[3][i] = d[i];
       orbmatr[2][i] = c[i];
     }
   orbmatr[1][1] = -su*cn-cusn*ci;
   orbmatr[1][2] = -su*sn+cucn*ci;
   orbmatr[1][3] = +cu*si;
 }
